/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.WebAppController");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.SecurityManager");
dojo.require("com.ibm.rational.report.web.core.app.internal.Session");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.rational.report.web.core.app.view.navigation.ReportTreeParser");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");

var ServiceabilityMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var InternalMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
var checkNotNull 				= net.jazz.ajax.toolbox.lang.checkNotNull;
var MessageFormatter			= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var getUserProfileLink			= com.ibm.rational.report.web.core.app.internal.Session.getUserProfileLink;
var ReportClient				= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session						= com.ibm.team.repository.web.client.session;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler		= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Main application controller for the Reports perspective.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.WebAppController",
	null, {

	/**
	 * Internal constructor. To be called only from WebAppView.
	 */
	constructor: function(webAppView) {
		this._msgFormatter = new MessageFormatter(null);
		this._view = webAppView;
		this._lastReportArgs = null;
		this._lastQueryArgs = null;
		this._lastSnapshotArgs = null;		
		
		// keep an instance of the security manager 
		this.secMgr = new com.ibm.rational.report.web.core.app.internal.SecurityManager(this);
		this.secMgr.init();
		
		//keep an instance of the report tree parser
		this.reportTreeParser = new com.ibm.rational.report.web.core.app.view.navigation.ReportTreeParser();
			
		// activate the Edit User Profile link 
		var self = this;
		var userProfileLink = getUserProfileLink();
		userProfileLink.onclick = function () {
			var args = new Object();
			args.userName = getAuthenticatedContributor().userId;
			args.editUserProfile = true;
			self.editUser(args);
		};
	},
	
	/**
	 * Called by Jazz web framework to provide default view when the URL
	 * contains no 'action' request parameter.
	 */
	init: function() {
	},
	
	// -------------- action API methods ----------------------------------------------------------
	
	/**
	 * Do nothing. Eventually, we'll want to display a default page.
	 */
	defaultAction: function() {
		this._view.clearMainContent();
	},
	
	/**
	 * View a report design.
	 * 
	 * @param args.name - the name of the report
	 */
	viewReportDesign: function(args) {
		checkNotNull(args.id, "Report id");
		args.type = "REPORT_DESIGN";
		this._view.showReport(args);
		this._lastReportArgs = args;
		this._lastQueryArgs = null;
		this._lastSnapshotArgs = null;
	},
	
	/**
	 * View a report query.
	 * 
	 * @param args.itemId - the item id of the query
	 */
	viewReportQuery: function(args) {
		checkNotNull(args.id, "Query id");
		args.type = "REPORT_QUERY";
		this._view.showReport(args);
		this._lastQueryArgs = args;
		this._lastReportArgs = null;
		this._lastSnapshotArgs = null;
	},
	
	/**
	 * View a report snapshot.
	 * 
	 * @param args.itemId - the item id of the snapshot
	 */
	viewReportSnapshot: function(args) {
		checkNotNull(args.id, "Snapshot id");
		args.type = "REPORT_SNAPSHOT";
		this._view.showReport(args);
		this._lastQueryArgs = null;
		this._lastReportArgs = null;
		this._lastSnapshotArgs = args;
	},
	
	/**
	 * Manage data sources.
	 */
	manageDataSources: function() {
		this._view.manageDataSources();
	},
	
	/**
	 * Create a new data source.
	 */
	createDataSource: function() {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.createDataSource();
	},
		
	/**
	 * Edit a data source.
	 */
	editDataSource: function(data) {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.editDataSource(data);
	},
	
	/**
	 * Manage reports.
	 */
	manageReports: function() {
		this._view.manageReports();
	},
	
	/**
	 * Upload a report design.
	 */
	newReportDesign: function(parentNodeId, postSave, postCancel) {
		this._view.newReportDesign(parentNodeId, postSave, postCancel);
	},
	
	/**
	 * Manage users and groups.
	 */
	manageUsers: function() {
		this._view.manageUsers();
	},
	
	/**
	 * Edit a user.
	 */
	editUser: function(args) {
		this._view.editUser(args.userName, args.editUserProfile);
	},
	
	/**
	 * Create a new user.
	 */
	newUser: function() {
		this._view.newUser();
	},
	
	/**
	 * View a user.
	 */
	viewUser: function(args) {
		this._view.viewUser(args.userName, args.editUserProfile);
	},

	/**
	 * Edit a user's password.
	 */
	editPassword: function(args) {
		this._view.editPassword(args.userName, args.editUserProfile);
	},
	
	
	/**
	 * Reset a user's password.
	 */
	resetPassword: function(args) {
		this._view.resetPassword(args.userName, args.editUserProfile);
	},
	
	/**
	 * View a group.
	 */	
	viewGroup: function(args) {
		this._view.viewGroup(args.groupName);
	},
	/**
	 * Edit a group.
	 */	
	editGroup: function(args) {
		this._view.editGroup(args.groupName);
	},
	
	/**
	 * Manage permissions.
	 */
	managePermissions: function() {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.managePermissions();
	},
	
	/**
	 * Edit the permissions for a resource.
	 */
	editPermissions: function(resource, data) {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.editPermissions(resource, data);
	},
	
	/**
	 * Assign users and groups to a resource.
	 */
	grantPermission: function(resource, principalMap, allPrincipals, aclEntryPrincipals, postSave, postCancel) {
		if (!this.secMgr.isProjectLead()) {
			return;
		}
		this._view.grantPermission(resource, principalMap, allPrincipals, aclEntryPrincipals, postSave, postCancel);
	},
	
	/**
	 * Manage projects.
	 */
	manageProjects: function() {
		this._view.manageProjects();
	},	
	
	/**
	 * Create a new project.
	 */
	createProject: function() {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.createProject();
	},
		
	/**
	 * Edit a project.
	 */
	editProject: function(data) {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.editProject(data);
	},
	
	/**
	 * Manage project members.
	 */
	editProjectMembers: function(project, data) {
		if (!this.secMgr.isAdmin()) {
			return;
		}
		this._view.editProjectMembers(project, data);
	},
	
	applySelectedTeamAreas: function() {
		if (this._lastReportArgs) {
			this.viewReportDesign(this._lastReportArgs);
		} else if (this._lastQueryArgs) {
			this.viewReportQuery(this._lastQueryArgs);
		} else if (this._lastSnapshotArgs) {
			this.viewReportSnapshot(this._lastSnapshotArgs);
		} else {
			this._displayErrorMessage(InternalMessages.SELECT_A_REPORT);
		}
	},
		
	newReportFolder: function(parentNode, postSave, postCancel) {
		if (this.secMgr.isGuest()) {
			return;
		}		
		this._view.newReportFolder(parentNode, postSave, postCancel);
	},

	/**
	 * Edit a report node or a report folder
	 */
	editReportNode: function(node, selectedReport, postSave, postCancel) {
		if (this.secMgr.isGuest()) {
			return;
		}
			
		this._view.editReportNode(node, selectedReport, postSave, postCancel);
	},
	
	/**
	 * Edit a report from report admin view
	 */
	editReport: function( selectedReport, postSave, postCancel) {
		if (!this.secMgr.isAdmin()) {
			return;
		}	
		this._view.editReportNode( null, selectedReport, postSave, postCancel);
	},
	
	addReportsToFolder: function(nodeId, postSave, postCancel) {
		if (this.secMgr.isGuest()) {
			return;
		}
		this._view.addReportsToFolder(nodeId, postSave, postCancel);
	},
		
	deleteReportNode: function(node, deleteReportItem, postSave, postCancel) {
		if (node != null && node.report) {
			var srh = new ServiceResponseHandler(this, "_postDeleteNodeSuccess", "_postDeleteReportError");
			ReportClient.postDeleteReportTreeNode(srh, { id: node.itemId, deleteReferencedReport: deleteReportItem});
		}
		else if(node != null){
			var srh = new ServiceResponseHandler(this, "_postDeleteNodeSuccess", "_postDeleteReportFolderError");
			ReportClient.postDeleteReportTreeNode(srh, { id: node.itemId, deleteReferencedReport: deleteReportItem});			
		}
	},
	
	_postDeleteNodeSuccess: function() {
		this.syncReportNodeChanged();
	},	
	
	_postDeleteReportError: function(error) {
		var msg = this._msgFormatter.formatErrorMessage(ServiceabilityMessages.FAILED_TO_DELETE_SELECTED_REPORT, error);
		this._displayErrorMessage(msg);
	},
		
	_postDeleteReportFolderError: function(error) {
		var msg = this._msgFormatter.formatErrorMessage(ServiceabilityMessages.FAILED_TO_DELETE_SELECTED_REPORT_FOLDER, error);
		this._displayErrorMessage(msg);
	},
	
	clearMainContent: function() {	
		this._view.clearMainContent();
	},
	
	getReportNavigator: function() {
		return this.sharedNavigator;
	},

	// ------------- initialization methods to be called from WebAppView ----------------------------
	
	initializeSharedReports: function(reportsNavigatorWidget) {
		checkNotNull(reportsNavigatorWidget, "reportsNavigatorWidget");
		this.sharedNavigator = reportsNavigatorWidget;
		this.sharedNavigator.displayReportNodes();
	},
	
	syncReportNodeChanged: function() {
		this.sharedNavigator.displayReportNodes();
	},
	
	_displayErrorMessage: function(errorMsg){
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}
	
});

})();